//+------------------------------------------------------------------+
//|                                                   DrawPivots.mq4 |
//|                                      Copyright  2006 Robert Dee |
//|                                         http://cashflowplans.net |
//+------------------------------------------------------------------+
#property copyright "Copyright  2006 Robert Dee"
#property link      "http://cashflowplans.net"
#property show_inputs

extern string prefix = "Day";  // set default for usual execution

//+------------------------------------------------------------------+
//| script program start function                                    |
//+------------------------------------------------------------------+
int start()
  {
//----
double o,h,l,c,range,pivot,r3,r2,r1,s1,s2,s3,m5,m4,m3,m2,m1,m0;
int shift,hour,o_shift,h_shift,l_shift,c_shift,o_time,c_time;

// clean up all old lines
ObjectDelete("_r3");
ObjectDelete("_r2");
ObjectDelete("_r1");
ObjectDelete("_pivot");
ObjectDelete("_s1");
ObjectDelete("_s2");
ObjectDelete("_s3");
ObjectDelete("_m5");
ObjectDelete("_m4");
ObjectDelete("_m3");
ObjectDelete("_m2");
ObjectDelete("_m1");
ObjectDelete("_m0");
//ObjectDelete("_open");
//ObjectDelete("_close");

if(prefix != "Day")  // assume that a month name is specified
   {
   o = iOpen(NULL,PERIOD_MN1,1);
   h = iHigh(NULL,PERIOD_MN1,1);
   l = iLow(NULL,PERIOD_MN1,1);
   c = iClose(NULL,PERIOD_MN1,1);
   Print("Last Month: ", o," ",  h," ",  l," ",  c);
   }
else  // usual daily path of execution - draw the daily pivots based on midnight New York Time
   {
   for(shift=0;shift<=48;shift++) // start with current bar and work backwards looking for 06:00
      {
      hour = TimeHour(iTime(NULL, PERIOD_H1, shift));
      if(hour == 6)
         {
         c_shift = shift;
         c_time = iTime(NULL, PERIOD_H1, c_shift);
         Print("Day Close: ", TimeToStr(c_time));
         //ObjectCreate("_close", OBJ_VLINE, 0, c_time, 0);
         //ObjectSet("_close", OBJPROP_COLOR, Teal);
         break;
         }
      }
   for(shift=shift+1;shift<=48;shift++) // start with current bar and work backwards looking for 06:00
      {
      hour = TimeHour(iTime(NULL, PERIOD_H1, shift));
      if(hour == 6)
         {
         o_shift = shift;
         o_time = iTime(NULL, PERIOD_H1, o_shift);
         Print("Day Open: ", TimeToStr(o_time));
         //ObjectCreate("_open", OBJ_VLINE, 0, o_time, 0);
         //ObjectSet("_open", OBJPROP_COLOR, Teal);
         break;
         }
      }
   h_shift = Highest(NULL, PERIOD_H1, MODE_HIGH, (o_shift - c_shift), c_shift); 
   l_shift = Lowest(NULL, PERIOD_H1, MODE_LOW, (o_shift - c_shift), c_shift); 
   o = iOpen(NULL,PERIOD_H1, o_shift);
   h = iHigh(NULL,PERIOD_H1, h_shift);
   l = iLow(NULL,PERIOD_H1, l_shift);
   c = iOpen(NULL,PERIOD_H1, c_shift);
   Print("Last Day: ", o," ",  h," ",  l," ",  c);
   prefix="";
   }

range = h - l;
pivot = (h + l + c )/3;

r3 = pivot + (range * 2);
r2 = pivot + range;
r1 = (2 * pivot) - l;
s1 = (2 * pivot) - h;
s2 = pivot - range;
s3 = pivot - (range * 2);

m5 = (r3 + r2) / 2;
m4 = (r2 + r1) / 2;
m3 = (r1 + pivot) / 2;
m2 = (pivot + s1) / 2;
m1 = (s1 + s2) / 2;
m0 = (s2 + s3) / 2;

ObjectCreate("_r3", OBJ_HLINE, 0, 0, r3);
ObjectCreate("_r2", OBJ_HLINE, 0, 0, r2);
ObjectCreate("_r1", OBJ_HLINE, 0, 0, r1);
ObjectCreate("_pivot", OBJ_HLINE, 0, 0, pivot);
ObjectCreate("_s1", OBJ_HLINE, 0, 0, s1);
ObjectCreate("_s2", OBJ_HLINE, 0, 0, s2);
ObjectCreate("_s3", OBJ_HLINE, 0, 0, s3);

ObjectCreate("_m5", OBJ_HLINE, 0, 0, m5);
ObjectCreate("_m4", OBJ_HLINE, 0, 0, m4);
ObjectCreate("_m3", OBJ_HLINE, 0, 0, m3);
ObjectCreate("_m2", OBJ_HLINE, 0, 0, m2);
ObjectCreate("_m1", OBJ_HLINE, 0, 0, m1);
ObjectCreate("_m0", OBJ_HLINE, 0, 0, m0);

ObjectSetText("_r3", prefix+"R3 "+DoubleToStr(r3,Digits));
ObjectSetText("_r2", prefix+"R2 "+DoubleToStr(r2,Digits));
ObjectSetText("_r1", prefix+"R1 "+DoubleToStr(r1,Digits));
ObjectSetText("_pivot", prefix+"PIVOT "+DoubleToStr(pivot,Digits)+" Range "+DoubleToStr(range,Digits));
ObjectSetText("_s1", prefix+"S1 "+DoubleToStr(s1,Digits));
ObjectSetText("_s2", prefix+"S2 "+DoubleToStr(s2,Digits));
ObjectSetText("_s3", prefix+"S3 "+DoubleToStr(s3,Digits));

ObjectSet("_r3", OBJPROP_COLOR, Green);
ObjectSet("_r2", OBJPROP_COLOR, Green);
ObjectSet("_r1", OBJPROP_COLOR, Green);
ObjectSet("_pivot", OBJPROP_COLOR, Purple);
ObjectSet("_s1", OBJPROP_COLOR, DodgerBlue);
ObjectSet("_s2", OBJPROP_COLOR, DodgerBlue);
ObjectSet("_s3", OBJPROP_COLOR, DodgerBlue);

ObjectSetText("_m5", prefix+"M5 "+DoubleToStr(m5,Digits));
ObjectSetText("_m4", prefix+"M4 "+DoubleToStr(m4,Digits));
ObjectSetText("_m3", prefix+"M3 "+DoubleToStr(m3,Digits));
ObjectSetText("_m2", prefix+"M2 "+DoubleToStr(m2,Digits));
ObjectSetText("_m1", prefix+"M1 "+DoubleToStr(m1,Digits));
ObjectSetText("_m0", prefix+"M0 "+DoubleToStr(m0,Digits));

ObjectSet("_m5", OBJPROP_COLOR, Green);
ObjectSet("_m4", OBJPROP_COLOR, Green);
ObjectSet("_m3", OBJPROP_COLOR, Green);
ObjectSet("_m2", OBJPROP_COLOR, DodgerBlue);
ObjectSet("_m1", OBJPROP_COLOR, DodgerBlue);
ObjectSet("_m0", OBJPROP_COLOR, DodgerBlue);


//----
   return(0);
  }
//+------------------------------------------------------------------+